using System;
using System.Collections.Generic;
using System.Linq;

class Program
{
  class Event {
    public string Name { get; private set; }
    public int StartHour { get; private set; }
    public int EndHour { get; private set; }
    public int Seats { get; private set; }
    
    public Event(string name, int start, int end, int seats)
    {
      Name = name;
      StartHour = start;
      EndHour = end;
      Seats = seats;
    }
    
    public void Schedule()
    {
      Seats--;
    }
    
    public bool ScheduleIfAvailable(int start)
    {
      if(StartHour >= start && Seats > 0)
      {
        Schedule();
        return true;
      }
      return false;
    }
  }

  static List<Event> CreateEvents()
  {
    return new List<Event>() {
      new Event("event 1", 10, 11, 10),
      new Event("event 2", 10, 11, 9),
      new Event("event 3", 14, 15, 0),
      new Event("event 4", 14, 15, 2),
      new Event("event 5", 15, 16, 7),
    };
  }

  static Event OldStyle(List<Event> events)
  {
    Event scheduledEvent = null;
    foreach(var anEvent in events)
    {
      if(anEvent.StartHour >= 12 && anEvent.Seats > 0)
      {
        anEvent.Schedule();
        scheduledEvent = anEvent;
        return anEvent;
      }
    }

    return null;
  }
  
  static Event BetterStyle(List<Event> events)
  {
    var pickedEvent = events.FirstOrDefault(anEvent => anEvent.StartHour >= 12 && anEvent.Seats > 0);
    if(pickedEvent != null) pickedEvent.Schedule();
    return pickedEvent;
  }

  static Event TellDontAskStyle(List<Event> events)
  {
    return events.FirstOrDefault(anEvent => anEvent.ScheduleIfAvailable(12));
  }
  
  static void Main()
  {
    //Schedule an event that's available in the after Noon.
    
    {
      Event scheduledEvent = OldStyle(CreateEvents());
      Console.WriteLine("You've been scheduled for the event: {0}", 
        scheduledEvent != null ? scheduledEvent.Name : "None");      
    }

    {
      Event scheduledEvent = BetterStyle(CreateEvents());
      Console.WriteLine("You've been scheduled for the event: {0}", 
        scheduledEvent != null ? scheduledEvent.Name : "None");      
    }

    {
      Event scheduledEvent = TellDontAskStyle(CreateEvents());
      Console.WriteLine("You've been scheduled for the event: {0}", 
        scheduledEvent != null ? scheduledEvent.Name : "None");      
    }
  }
}
